/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef nsLSParser_h__
#define nsLSParser_h__

#include "nsIDOMLSParser.h"
#include "nsIDOMEventTarget.h"
#include "nsIXMLHttpRequest.h"

#include "nsCOMPtr.h"
#include "nsCOMArray.h"

class nsIDOMLSParserFilter;


#define NS_LSPARSER_CID                               \
  { /* 7956d067-dfab-43c1-a11c-c7b10e0d8dca */        \
    0x7956d067, 0xdfab, 0x43c1,                       \
   {0xa1, 0x1c, 0xc7, 0xb1, 0x0e, 0x0d, 0x8d, 0xca} }

#define NS_LSPARSER_CONTRACTID \
  "@mozilla.org/dom/lsparser;1"


class nsLSParser : public nsIDOMLSParser,
                   public nsIDOMEventTarget
{
public: 
  nsLSParser();
  virtual ~nsLSParser();

  NS_DECL_ISUPPORTS

  // nsIDOMLSParser
  NS_DECL_NSIDOMLSPARSER

  // nsIDOMEventTarget
  NS_DECL_NSIDOMEVENTTARGET

  nsresult Init();

  void FireOnLoad();

  nsIXMLHttpRequest *XMLHttpRequest()
  {
    return mXMLHttpRequest;
  }

private:
  nsCOMPtr<nsIXMLHttpRequest> mXMLHttpRequest;
  nsCOMPtr<nsIDOMLSParserFilter> mFilter;

  nsCOMArray<nsIDOMEventListener> mLoadListeners;
  nsCOMArray<nsIDOMEventListener> mProgressListeners;

  PRPackedBool mIsAsync;
};

#endif
