/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Netscape security libraries.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1994-2000
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#define MACI_ChangePIN			NSCI_ChangePIN
#define MACI_CheckPIN			NSCI_CheckPIN
#define MACI_Close			NSCI_Close
#define MACI_Decrypt			NSCI_Decrypt
#define MACI_DeleteCertificate		NSCI_DeleteCertificate
#define MACI_DeleteKey			NSCI_DeleteKey
#define MACI_Encrypt			NSCI_Encrypt
#define MACI_ExtractX 			NSCI_ExtractX
#define MACI_FirmwareUpdate 		NSCI_FirmwareUpdate
#define MACI_GenerateIV			NSCI_GenerateIV
#define MACI_GenerateMEK		NSCI_GenerateMEK
#define MACI_GenerateRa			NSCI_GenerateRa
#define MACI_GenerateRandom		NSCI_GenerateRandom
#define MACI_GenerateTEK 		NSCI_GenerateTEK
#define MACI_GenerateX 			NSCI_GenerateX
#define MACI_GetCertificate		NSCI_GetCertificate
#define MACI_GetConfiguration		NSCI_GetConfiguration
#define MACI_GetHash			NSCI_GetHash
#define MACI_GetPersonalityList		NSCI_GetPersonalityList
#define MACI_GetSessionID		NSCI_GetSessionID
#define MACI_GetState			NSCI_GetState
#define MACI_GetStatus			NSCI_GetStatus
#define MACI_GetTime			NSCI_GetTime
#define MACI_Hash			NSCI_Hash
#define MACI_Initialize			NSCI_Initialize
#define MACI_InitializeHash		NSCI_InitializeHash
#define MACI_InstallX 			NSCI_InstallX
#define MACI_LoadCertificate 		NSCI_LoadCertificate
#define MACI_LoadDSAParameters	 	NSCI_LoadDSAParameters
#define MACI_LoadInitValues		NSCI_LoadInitValues
#define MACI_LoadIV			NSCI_LoadIV
#define MACI_LoadX 			NSCI_LoadX
#define MACI_Lock			NSCI_Lock
#define MACI_Open			NSCI_Open
#define MACI_RelayX 			NSCI_RelayX
#define MACI_Reset			NSCI_Reset
#define MACI_Restore			NSCI_Restore
#define MACI_Save			NSCI_Save
#define MACI_Select			NSCI_Select
#define MACI_SetConfiguration		NSCI_SetConfiguration
#define MACI_SetKey			NSCI_SetKey
#define MACI_SetMode			NSCI_SetMode
#define MACI_SetPersonality		NSCI_SetPersonality
#define MACI_SetTime			NSCI_SetTime
#define MACI_Sign			NSCI_Sign
#define MACI_Terminate			NSCI_Terminate
#define MACI_TimeStamp			NSCI_TimeStamp
#define MACI_Unlock			NSCI_Unlock
#define MACI_UnwrapKey			NSCI_UnwrapKey
#define MACI_VerifySignature		NSCI_VerifySignature
#define MACI_VerifyTimeStamp		NSCI_VerityTimeStap
#define MACI_WrapKey			NSCI_WrapKey
#define MACI_Zeroize			NSCI_Zeroize

